set(header "Python wheel (${Python3_VERSION}):")

# Python package directory structure:
#
# py  {Python package build-workspace}
# |...{Package setup config files}
# +--src
# |  |
# |  +--bornagain
# |     |...{Package init files}
# |     +--lib  {BornAgain libraries and their Python wrappers}
# |        |
# |        +--extra  {Extra dependencies}
# |
# +--wheel  {wheel for the Python version}

set(WHEEL_ROOT_DIR "${CMAKE_SOURCE_DIR}/Wrap/Python")

set(WHEEL_C_EXT_INC_DIRS "${Python3_INCLUDE_DIRS}")
set(WHEEL_C_EXT_LIB_DIRS "${Python3_LIBRARY_DIRS}")

# output directories:

set(PY_OUTPUT_DIR "${CMAKE_BINARY_DIR}/py")
set(WHEEL_DIR "${PY_OUTPUT_DIR}/wheel")
set(BA_PY_SOURCE_OUTPUT_DIR "${PY_OUTPUT_DIR}/src")
set(BA_PY_INIT_OUTPUT_DIR "${PY_OUTPUT_DIR}/src/bornagain")
set(BA_PY_LIBRARY_OUTPUT_DIR "${PY_OUTPUT_DIR}/src/bornagain/lib")
set(BA_PY_EXTRA_LIBRARY_OUTPUT_DIR "${PY_OUTPUT_DIR}/src/bornagain/lib/extra")

# make the directories

file(MAKE_DIRECTORY
    "${PY_OUTPUT_DIR}"
    "${WHEEL_DIR}"
    "${BA_PY_SOURCE_OUTPUT_DIR}"
    "${BA_PY_INIT_OUTPUT_DIR}"
    "${BA_PY_LIBRARY_OUTPUT_DIR}"
    "${BA_PY_EXTRA_LIBRARY_OUTPUT_DIR}"
    )

# configure setup files

set(_setup_cfg ${PY_OUTPUT_DIR}/setup.cfg)
set(_setup_py ${PY_OUTPUT_DIR}/setup.py)

configure_file(${WHEEL_ROOT_DIR}/setup.cfg.in ${_setup_cfg} @ONLY)
configure_file(${WHEEL_ROOT_DIR}/setup.py.in ${_setup_py} @ONLY)
configure_file(${WHEEL_ROOT_DIR}/pyproject.toml ${PY_OUTPUT_DIR} COPYONLY)

# copy text files

configure_file(${CMAKE_SOURCE_DIR}/README.md ${PY_OUTPUT_DIR} COPYONLY)
configure_file(${CMAKE_SOURCE_DIR}/COPYING ${PY_OUTPUT_DIR} COPYONLY)
configure_file(${CMAKE_SOURCE_DIR}/CITATION ${PY_OUTPUT_DIR} COPYONLY)

# dummy C extension for the Python package
# NOTE: Dummy C extension is used merely to set the correct wheel tag according to PEP 425

configure_file("${WHEEL_ROOT_DIR}/src/bornagain_c_ext_dummy.c" ${BA_PY_SOURCE_OUTPUT_DIR} COPYONLY)

# store init files for the Python package

configure_file(${WHEEL_ROOT_DIR}/src/bornagain/__init__.py.in
    ${BA_PY_INIT_OUTPUT_DIR}/__init__.py @ONLY)

file(GLOB _sources ${WHEEL_ROOT_DIR}/src/bornagain/*.py)
foreach(_src ${_sources})
    configure_file(${_src} ${BA_PY_INIT_OUTPUT_DIR})
endforeach()
file(GLOB _sources ${WHEEL_ROOT_DIR}/src/bornagain/lib/*.py)
foreach(_src ${_sources})
    configure_file(${_src} ${BA_PY_LIBRARY_OUTPUT_DIR})
endforeach()

# external library dependencies

if(WIN32)
    set(_extra_libs "${BA_Dependencies_WIN32}")
else()
    set(_extra_libs "${BA_Dependencies}")
endif()

foreach(_src ${_extra_libs})
    configure_file(${_src} ${BA_PY_EXTRA_LIBRARY_OUTPUT_DIR} COPYONLY)
endforeach()

##... target BAPyWheel

if(APPLE)
    # On MacOS, building the Python packages needs further effort
    # which is performed in a dedicated shell script which
    # should be executed after build process is done (like CPack).
    set(MACPKG_EXTRA_LIBS "${_extra_libs}")

    set(BA_PY_MODE "ON")
    configure_file("${CMAKE_SOURCE_DIR}/devtools/deploy/mac/mac_package.py.in"
        ${BUILD_VAR_DIR}/mac_py_package.py @ONLY)
    unset(BA_PY_MODE)

    add_custom_target(BAPyWheel ALL
        COMMENT "${header} Script to build the Python wheel: "
        "'${${BUILD_VAR_DIR}/mk_pypack_macos.zsh}'"
        )
elseif(LINUX)
    # On Linux, building the Python packages needs further effort
    # which is performed in a dedicated shell script which
    # should be executed after build process is done (like CPack).
    configure_file("${CMAKE_SOURCE_DIR}/devtools/deploy/linux/mk_wheel_multilinux.sh.in"
        ${BUILD_VAR_DIR}/mk_wheel_multilinux.sh @ONLY)
    add_custom_target(BAPyWheel ALL
        COMMENT "${header} Script to build the Python wheel: "
        "'${${BUILD_VAR_DIR}/mk_wheel_multilinux.sh}'"
        )
else()
    # MS-Windows
    add_custom_target(BAPyWheel ALL
        COMMAND ${Python3_EXECUTABLE} -m pip wheel ${PY_OUTPUT_DIR} --no-deps --wheel ${WHEEL_DIR}
        COMMENT "${header} Making the Python wheel..."
        )
endif()
